from trac.core import *
from trac.web.api import IRequestFilter
from trac.config import ListOption
from trac.web.chrome import INavigationContributor, ITemplateProvider
from trac.util import Markup

try:
    set = set
except:
    from sets import Set as set

__all__ = ['NavHiderModule']

class NavHiderModule(Component):
    """Request filter to hide entries in navigation bars."""

    mainnav = ListOption('navhider', 'mainnav',
                              doc='Items to remove from the mainnav bar')

    metanav = ListOption('navhider', 'metanav',
                              doc='Items to remove from the metanav bar')
                             
    implements(IRequestFilter)
    implements(INavigationContributor)

    nav_contributors = ExtensionPoint(INavigationContributor)

    # IRequestFilter methods
    def pre_process_request(self, req, handler):
        return handler
        
    def post_process_request(self, req, template, content_type):
        self._remove_items(req, 'mainnav')
        self._remove_items(req, 'metanav')    
        return template, content_type

    # Internal methods
    def _remove_items(self, req, name):
        items = set(getattr(self, name))
        for item in self.config.get('trac', name, default='').split(','):
            item = item.strip()
            if item[0] == '-' or item[0] == '!':
                items.add(item[1:])
        for item in items:
            req.hdf.removeTree('chrome.nav.%s.%s'%(name,item))
        
    # INavigationContributor methods
    def get_active_navigation_item(self, req):
        return ''
                
    def get_navigation_items(self, req):
	add = self.env.config.get('navadd', 'add_items', ''). \
		replace(',', ' ').split()
	
	items = []
	for a in add:
	    title = self.env.config.get('navadd', '%s.title' % a)
	    url = self.env.config.get('navadd', '%s.url' % a)
	    perm = self.env.config.get('navadd', '%s.perm' % a)
	    target = self.env.config.get('navadd', '%s.target' % a)

	    if perm and not req.perm.has_permission(perm):
		continue

	    if target not in ('mainnav', 'metanav'):
		target = 'mainnav'

	    items.append((target, a, Markup('<a href="%s">%s</a>' % (url, title))))
	
	return items
